@@ -1,8 +1,10 @@
+import itertools
 from retry import retry
 from retry.api import retry_call
 from dataclasses import asdict
 import requests
 from packaging.version import parse as parse_version
+from time import sleep
 
 from uiautomator2.core import HTTPResponse, _http_request
 from kea2.adbUtils import ADBDevice, ADBStreamShell_V2
@@ -28,6 +30,7 @@ class FastbotManager:
         ADBDevice.setDevice(options.serial, options.transport_id)
         self.dev = ADBDevice()
         self.android_release = parse_version(self.dev.getprop("ro.build.version.release"))
+        self.executed_prop = False
 
     def _activateFastbot(self) -> ADBStreamShell_V2:
         """
@@ -100,7 +103,9 @@ class FastbotManager:
             _http_request(dev=self.dev, device_port=8090, method="GET", path="/ping")
 
         try:
-            retry_call(_check_alive_request, tries=10, delay=2)
+            logger.info("Connecting to fastbot server...")
+            retry_call(_check_alive_request, tries=10, delay=2, logger=logger)
+            logger.info("Connected to fastbot server.")
         except requests.ConnectionError:
             raise RuntimeError("Failed to connect fastbot")
 
@@ -111,7 +116,8 @@ class FastbotManager:
     def init(self, options: "Options", stamp):
         post_data = {
             "takeScreenshots": options.take_screenshots,
-            "Stamp": stamp,
+            "preFailureScreenshots": options.pre_failure_screenshots,
+            "logStamp": stamp,
             "deviceOutputRoot": options.device_output_root,
         }
         r = _http_request(
@@ -161,6 +167,15 @@ class FastbotManager:
         res = r.text
         if res != "OK":
             print(f"[ERROR] Error when logging script: {execution_info}", flush=True)
+    
+    @retry(Exception, tries=2, delay=2)
+    def dumpHierarchy(self):
+        sleep(self.options.throttle / 1000)
+        r = self.request(
+            method="GET",
+            path="/dumpHierarchy",
+        )
+        return r.json()['result']
 
     @property
     def device_output_dir(self):
@@ -173,10 +188,8 @@ class FastbotManager:
             "/sdcard/framework.jar:"
             "/sdcard/fastbot-thirdpart.jar:"
             "/sdcard/kea2-thirdpart.jar",
-
             "exec", "app_process",
             "/system/bin", "com.android.commands.monkey.Monkey",
-            "-p", *self.options.packageNames,
             "--agent-u2" if self.options.agent == "u2" else "--agent",
             "reuseq",
             "--running-minutes", f"{self.options.running_mins}",
@@ -185,6 +198,9 @@ class FastbotManager:
             "--output-directory", f"{self.options.device_output_root}/output_{self.options.log_stamp}",
         ]
 
+        pkgs = itertools.chain.from_iterable(["-p", pkg] for pkg in self.options.packageNames)
+        shell_command.extend(pkgs)
+
         if self.options.profile_period:
             shell_command += ["--profile-period", f"{self.options.profile_period}"]
 
