@@ -13,7 +13,7 @@ class ResultSyncer:
 
     def __init__(self, device_output_dir, options: "Options"):
         self.device_output_dir = device_output_dir
-        self.output_dir = Path(options.output_dir) / Path(device_output_dir).name
+        self.output_dir = options.output_dir / Path(device_output_dir).name
         self.running = False
         self.thread = None
         self.sync_event = threading.Event()
